
__device__ __forceinline__ uint bfi(uint x, uint y, uint bit, uint numBits) {
	uint ret;
	asm("bfi.b32 %0, %1, %2, %3, %4;" :
		"=r"(ret) : "r"(y), "r"(x), "r"(bit), "r"(numBits));
	return ret;
}

#define WARP_SIZE 32

__global__ void BallotScanWarp(const float *dataIn_global, float *dataOut_global, uint *countOut_global) {
	uint tid = threadIdx.x;
	float val = dataIn_global[tid];

	uint flag = -1.0f != val;

	uint bits = __ballot(flag);

	uint mask = bfi(0, 0xffffffff, 0, tid);
	uint exc = __popc(mask & bits);
	uint total = __popc(bits);

	__shared__ volatile float shared[WARP_SIZE];
	if (flag)
		shared[exc] = val;
	val = shared[tid];

	if (tid < total)
		dataOut_global[tid] = val;

	if (!tid) *countOut_global = total;
}
