
#define WARP_SIZE 32

__global__ void warp_scan1(const int *values, int *inclusive, int *exclusive) {
	__shared__ volatile int scan[WARP_SIZE];
	int tid = threadIdx.x;
	int lane = (WARP_SIZE-1) & tid;

	// read from global
	int x = values[lane];
	scan[lane] = x;

	int sum = x;
	#pragma unroll
	for (int offset=1; offset<WARP_SIZE; offset*=2) {
		if (lane>=offset) sum += scan[lane-offset];
		scan[lane] = sum;
	}
	inclusive[lane] = sum;
	exclusive[lane] = sum - x;
}

#define SCAN_STRIDE (WARP_SIZE + WARP_SIZE/2)

__global__ void warp_scan2(const int *values, int *inclusive, int *exclusive) {
	__shared__ volatile int scan[SCAN_STRIDE];
	int tid = threadIdx.x;
	int lane = (WARP_SIZE-1) & tid;

	volatile int *s = scan + lane + WARP_SIZE/2;
	s[-(WARP_SIZE/2)] = 0;

	// Read from global memory.
	int x = values[lane];
	s[0] = x;

	// Run each pass of the scan.
	int sum = x;
	for (int offset=1; offset<WARP_SIZE; offset*=2) {
		sum += s[-offset];
		s[0] = sum;
	}

	// Write sum to inclusive and sum - x (the original source element) to
	// exclusive.
	inclusive[lane] = sum;
	exclusive[lane] = sum - x;
}
